<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<MetadataContainerType xmlns="http://textgrid.info/namespaces/metadata/core/2010" xmlns:ns2="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
    <object>
        <generic>
            <provided>
                <title>{{title}}</title>
                <format>{{format}}</format>
            </provided>
        </generic>
    </object>
</MetadataContainerType>
